unit FileList;

{$IMPORTEDDATA OFF}

interface

type
  TFileList = class(TObject)
  public
    Cur: Integer;
    Count: Integer;
    Items: array of WideString;
    Positions: array of Cardinal; // for archive plugin
    constructor Create;
    procedure Add(const Path: WideString; Position: Cardinal = 0);
    procedure AddDir(const Path: WideString);
    procedure Clear;
    procedure Delete(Index: Integer);
    procedure Sort;
  end;

implementation

uses NLib, NSys, NCmpLogical;

constructor TFileList.Create;
begin
  Clear;
end;

procedure TFileList.Add(const Path: WideString; Position: Cardinal = 0);
begin
  if Count = Length(Items) then begin
    SetLength(Items, Count + Count div 4);
    SetLength(Positions, Count + Count div 4);
  end;
  Items[Count] := Path;
  Positions[Count] := Position;
  Inc(Count);
end;

procedure TFileList.AddDir(const Path: WideString);
var
  dir: TDir;
begin
  with TListDir.Create(Path) do
    try
      while Each(dir) do
        if not (dmDirectory in dir.Mode) then
          Add(JoinPath([Path, dir.Name]));
    finally
      Free;
    end;
end;

procedure TFileList.Clear;
begin
  Count := 0;
  SetLength(Items, 32);
  SetLength(Positions, 32);
end;

procedure TFileList.Delete(Index: Integer);
var
  i: Integer;
begin
  if (Index < 0) or (Index >= Count) then
    Exit;
  Dec(Count);
  for i := Index to Count - 1 do begin
    Items[i] := Items[i+1];
    Positions[i] := Positions[i+1];
  end;
end;

procedure TFileList.Sort;
  procedure merge(
      var SrcItems, DestItems: array of WideString;
      var SrcPositions, DestPositions: array of Cardinal;
      L, H: Integer);
  var
    i, j, k, m: Integer;
  begin
    if L >= H then
      Exit;
    m := (H + L) div 2;
    merge(DestItems, SrcItems, DestPositions, SrcPositions, L, m);
    merge(DestItems, SrcItems, DestPositions, SrcPositions, m+1, H);
    j := L;
    k := m+1;
    for i := L to H do begin
      if (k > H) or (j <= m) and (CmpLogical(SrcItems[j], SrcItems[k]) <= 0) then begin
        DestItems[i] := SrcItems[j];
        DestPositions[i] := SrcPositions[j];
        Inc(j);
      end else begin
        DestItems[i] := SrcItems[k];
        DestPositions[i] := SrcPositions[k];
        Inc(k);
      end;
    end;
  end;

var
  i: Integer;
  auxitems: array of WideString;
  auxpositions: array of Cardinal;
begin
  SetLength(auxitems, Count);
  SetLength(auxpositions, Count);
  for i := 0 to Count-1 do begin
    auxitems[i] := Items[i];
    auxpositions[i] := Positions[i];
  end;
  merge(auxitems, Items, auxpositions, Positions, 0, High(auxitems));
end;

end.
